import datetime
from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.allow import Allow
from ...models.auth_error_response_schema import AuthErrorResponseSchema
from ...models.avoid import Avoid
from ...models.driver import Driver
from ...models.ev_empirical_model import EVEmpiricalModel
from ...models.ev_physical_model import EVPhysicalModel
from ...models.exclude import Exclude
from ...models.fuel import Fuel
from ...models.return_ import Return
from ...models.router_mode import RouterMode
from ...models.router_route_response import RouterRouteResponse
from ...models.routing_error_response import RoutingErrorResponse
from ...models.routing_mode import RoutingMode
from ...models.scooter import Scooter
from ...models.spans import Spans
from ...models.taxi import Taxi
from ...models.tolls import Tolls
from ...models.traffic import Traffic
from ...models.truck import Truck
from ...models.units import Units
from ...models.vehicle import Vehicle
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    transport_mode: RouterMode,
    origin: str,
    destination: str,
    via: list[str] | Unset = UNSET,
    departure_time: str | Unset = UNSET,
    arrival_time: datetime.datetime | Unset = UNSET,
    routing_mode: RoutingMode | Unset = UNSET,
    alternatives: int | Unset = 0,
    avoid: Avoid | Unset = UNSET,
    allow: Allow | Unset = UNSET,
    exclude: Exclude | Unset = UNSET,
    units: Units | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    return_: list[Return] | Unset = UNSET,
    spans: list[Spans] | Unset = UNSET,
    truck: Truck | Unset = UNSET,
    vehicle: Vehicle | Unset = UNSET,
    consumption_model: str | Unset = UNSET,
    ev: EVEmpiricalModel | EVPhysicalModel | Unset = UNSET,
    fuel: Fuel | Unset = UNSET,
    driver: Driver | Unset = UNSET,
    pedestrianspeed: float | Unset = UNSET,
    scooter: Scooter | Unset = UNSET,
    currency: str | Unset = UNSET,
    customizations: list[str] | Unset = UNSET,
    taxi: Taxi | Unset = UNSET,
    tolls: Tolls | Unset = UNSET,
    max_speed_on_segment: str | Unset = UNSET,
    traffic: Traffic | Unset = UNSET,
    billing_tag: str | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}
    if not isinstance(x_request_id, Unset):
        headers["X-Request-ID"] = x_request_id

    params: dict[str, Any] = {}

    json_transport_mode = transport_mode.value
    params["transportMode"] = json_transport_mode

    params["origin"] = origin

    params["destination"] = destination

    json_via: list[str] | Unset = UNSET
    if not isinstance(via, Unset):
        json_via = via

    params["via"] = json_via

    params["departureTime"] = departure_time

    json_arrival_time: str | Unset = UNSET
    if not isinstance(arrival_time, Unset):
        json_arrival_time = arrival_time.isoformat()
    params["arrivalTime"] = json_arrival_time

    json_routing_mode: str | Unset = UNSET
    if not isinstance(routing_mode, Unset):
        json_routing_mode = routing_mode.value

    params["routingMode"] = json_routing_mode

    params["alternatives"] = alternatives

    json_avoid: dict[str, Any] | Unset = UNSET
    if not isinstance(avoid, Unset):
        json_avoid = avoid.to_dict()
    if not isinstance(json_avoid, Unset):
        params.update((f"avoid[{k}]", v) for k, v in json_avoid.items())

    json_allow: dict[str, Any] | Unset = UNSET
    if not isinstance(allow, Unset):
        json_allow = allow.to_dict()
    if not isinstance(json_allow, Unset):
        params.update((f"allow[{k}]", v) for k, v in json_allow.items())

    json_exclude: dict[str, Any] | Unset = UNSET
    if not isinstance(exclude, Unset):
        json_exclude = exclude.to_dict()
    if not isinstance(json_exclude, Unset):
        params.update((f"exclude[{k}]", v) for k, v in json_exclude.items())

    json_units: str | Unset = UNSET
    if not isinstance(units, Unset):
        json_units = units.value

    params["units"] = json_units

    json_lang: list[str] | Unset = UNSET
    if not isinstance(lang, Unset):
        json_lang = lang

    params["lang"] = json_lang

    json_return_: list[str] | Unset = UNSET
    if not isinstance(return_, Unset):
        json_return_ = []
        for return_item_data in return_:
            return_item = return_item_data.value
            json_return_.append(return_item)

    params["return"] = json_return_

    json_spans: list[str] | Unset = UNSET
    if not isinstance(spans, Unset):
        json_spans = []
        for spans_item_data in spans:
            spans_item = spans_item_data.value
            json_spans.append(spans_item)

    params["spans"] = json_spans

    json_truck: dict[str, Any] | Unset = UNSET
    if not isinstance(truck, Unset):
        json_truck = truck.to_dict()
    if not isinstance(json_truck, Unset):
        params.update((f"truck[{k}]", v) for k, v in json_truck.items())

    json_vehicle: dict[str, Any] | Unset = UNSET
    if not isinstance(vehicle, Unset):
        json_vehicle = vehicle.to_dict()
    if not isinstance(json_vehicle, Unset):
        params.update((f"vehicle[{k}]", v) for k, v in json_vehicle.items())

    params["consumptionModel"] = consumption_model

    json_ev: dict[str, Any] | Unset
    if isinstance(ev, Unset):
        json_ev = UNSET
    elif isinstance(ev, EVEmpiricalModel):
        json_ev = ev.to_dict()
    else:
        json_ev = ev.to_dict()

    params["ev"] = json_ev

    json_fuel: dict[str, Any] | Unset = UNSET
    if not isinstance(fuel, Unset):
        json_fuel = fuel.to_dict()
    if not isinstance(json_fuel, Unset):
        params.update((f"fuel[{k}]", v) for k, v in json_fuel.items())

    json_driver: dict[str, Any] | Unset = UNSET
    if not isinstance(driver, Unset):
        json_driver = driver.to_dict()
    if not isinstance(json_driver, Unset):
        params.update((f"driver[{k}]", v) for k, v in json_driver.items())

    params["pedestrian[speed]"] = pedestrianspeed

    json_scooter: dict[str, Any] | Unset = UNSET
    if not isinstance(scooter, Unset):
        json_scooter = scooter.to_dict()
    if not isinstance(json_scooter, Unset):
        params.update((f"scooter[{k}]", v) for k, v in json_scooter.items())

    params["currency"] = currency

    json_customizations: list[str] | Unset = UNSET
    if not isinstance(customizations, Unset):
        json_customizations = customizations

    params["customizations"] = json_customizations

    json_taxi: dict[str, Any] | Unset = UNSET
    if not isinstance(taxi, Unset):
        json_taxi = taxi.to_dict()
    if not isinstance(json_taxi, Unset):
        params.update((f"taxi[{k}]", v) for k, v in json_taxi.items())

    json_tolls: dict[str, Any] | Unset = UNSET
    if not isinstance(tolls, Unset):
        json_tolls = tolls.to_dict()
    if not isinstance(json_tolls, Unset):
        params.update((f"tolls[{k}]", v) for k, v in json_tolls.items())

    params["maxSpeedOnSegment"] = max_speed_on_segment

    json_traffic: dict[str, Any] | Unset = UNSET
    if not isinstance(traffic, Unset):
        json_traffic = traffic.to_dict()
    if not isinstance(json_traffic, Unset):
        params.update((f"traffic[{k}]", v) for k, v in json_traffic.items())

    params["billingTag"] = billing_tag

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/routes",
        "params": params,
    }

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> (
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | RoutingErrorResponse
    | RouterRouteResponse
    | RoutingErrorResponse
    | None
):
    if response.status_code == 200:
        response_200 = RouterRouteResponse.from_dict(response.json())

        return response_200

    if response.status_code == 400:
        response_400 = RoutingErrorResponse.from_dict(response.json())

        return response_400

    if response.status_code == 401:
        response_401 = AuthErrorResponseSchema.from_dict(response.json())

        return response_401

    if response.status_code == 403:

        def _parse_response_403(
            data: object,
        ) -> AuthErrorResponseSchema | RoutingErrorResponse:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_routing_403_error_response_schema_type_0 = (
                    AuthErrorResponseSchema.from_dict(data)
                )

                return componentsschemas_routing_403_error_response_schema_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_routing_403_error_response_schema_type_1 = (
                RoutingErrorResponse.from_dict(data)
            )

            return componentsschemas_routing_403_error_response_schema_type_1

        response_403 = _parse_response_403(response.json())

        return response_403

    if response.status_code == 500:
        response_500 = RoutingErrorResponse.from_dict(response.json())

        return response_500

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | RoutingErrorResponse
    | RouterRouteResponse
    | RoutingErrorResponse
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    transport_mode: RouterMode,
    origin: str,
    destination: str,
    via: list[str] | Unset = UNSET,
    departure_time: str | Unset = UNSET,
    arrival_time: datetime.datetime | Unset = UNSET,
    routing_mode: RoutingMode | Unset = UNSET,
    alternatives: int | Unset = 0,
    avoid: Avoid | Unset = UNSET,
    allow: Allow | Unset = UNSET,
    exclude: Exclude | Unset = UNSET,
    units: Units | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    return_: list[Return] | Unset = UNSET,
    spans: list[Spans] | Unset = UNSET,
    truck: Truck | Unset = UNSET,
    vehicle: Vehicle | Unset = UNSET,
    consumption_model: str | Unset = UNSET,
    ev: EVEmpiricalModel | EVPhysicalModel | Unset = UNSET,
    fuel: Fuel | Unset = UNSET,
    driver: Driver | Unset = UNSET,
    pedestrianspeed: float | Unset = UNSET,
    scooter: Scooter | Unset = UNSET,
    currency: str | Unset = UNSET,
    customizations: list[str] | Unset = UNSET,
    taxi: Taxi | Unset = UNSET,
    tolls: Tolls | Unset = UNSET,
    max_speed_on_segment: str | Unset = UNSET,
    traffic: Traffic | Unset = UNSET,
    billing_tag: str | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> Response[
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | RoutingErrorResponse
    | RouterRouteResponse
    | RoutingErrorResponse
]:
    """Calculate routes via GET

     Calculates a route using a generic vehicle/pedestrian mode, e.g. car, truck, pedestrian, etc...

    Args:
        transport_mode (RouterMode): Mode of transport to be used for route calculation.
        origin (str): A location defining an origin, destination or via point for a route or an
            isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
            * `charging`: Structured string denoting a user-planned charging stop.

                Format: `charging=(power=<value>;current=<value>;voltage=<value>;supplyType=<value>;mi
            nDuration=<value>;maxDuration=<value>)`.

                The properties `power`, `current`, `voltage` and `supplyType` denote the
            characteristics of the chosen compatible connector at the station. These are all required.
                `minDuration` and `maxDuration` set the time bounds for the charging time. At least
            one of them is required. For most use cases we recommend to provide at least
            `minDuration`.
                The following are the specifications for the properties:
                + `power`:        value in kW, type: number. Rated power of the connector.
                + `voltage`:      value in V, type: number. Rated voltage of the connector.
                + `current`:      value in A, type: number. Rated current of the connector.
                + `supplyType`:   one of {"acSingle", "acThree", "dc"} for 1-phase AC, 3-phase AC and
            DC respectively, type: string.
                + `minDuration`:  value in seconds, type: int. Minimum time the user expects to charge
            at the station, including `chargingSetupDuration`.
                + `maxDuration`:  value in seconds, type: int. Maximum time the user plans to charge
            at the station, including `chargingSetupDuration`.

                For a user-planned charging stop, the following properties of the `ev` parameter are
            also required (see documentation for the `ev` parameter):
                + `initialCharge`
                + `maxCharge`
                + `chargingCurve`

                  *Notes*:
                  * This option is only supported for /v8/routes endpoint.
                  * This option is not supported for pass-through waypoints.
                  * If `makeReachable=true` and `minDuration` is not provided (or if `minDuration=0`),
            route calculation may suggest not to charge on this station, for example,
                    if a better station is available in the vicinity.
                  * If `stopDuration` is provided then the total time at the stop is the higher of
            (`stopDuration`, `chargingSetupDuration` + `chargingDuration`).
                  * If charging for `minDuration` would charge above `maxCharge`, the time spent
            charging is capped by `maxCharge`.
                  * If `stopDuration` or `minDuration` exceed `chargingSetupDuration` +
            `chargingDuration`, the `waiting` post action duration is set to the remaining time.
                  * If `makeReachable=true` is not set, the target charge is
            `maxChargeAfterChargingStation`, unless the charging time would be outside of the
            specified duration range.
                    In such case, the nearest valid value is used.
                  * In getRoutesByHandle requests, the target charge from the original route response
            is used, unless the charging time would be outside of the specified duration range.
                    In such case, the nearest valid value is used.
        destination (str): A location defining an origin, destination or via point for a route or
            an isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
            * `charging`: Structured string denoting a user-planned charging stop.

                Format: `charging=(power=<value>;current=<value>;voltage=<value>;supplyType=<value>;mi
            nDuration=<value>;maxDuration=<value>)`.

                The properties `power`, `current`, `voltage` and `supplyType` denote the
            characteristics of the chosen compatible connector at the station. These are all required.
                `minDuration` and `maxDuration` set the time bounds for the charging time. At least
            one of them is required. For most use cases we recommend to provide at least
            `minDuration`.
                The following are the specifications for the properties:
                + `power`:        value in kW, type: number. Rated power of the connector.
                + `voltage`:      value in V, type: number. Rated voltage of the connector.
                + `current`:      value in A, type: number. Rated current of the connector.
                + `supplyType`:   one of {"acSingle", "acThree", "dc"} for 1-phase AC, 3-phase AC and
            DC respectively, type: string.
                + `minDuration`:  value in seconds, type: int. Minimum time the user expects to charge
            at the station, including `chargingSetupDuration`.
                + `maxDuration`:  value in seconds, type: int. Maximum time the user plans to charge
            at the station, including `chargingSetupDuration`.

                For a user-planned charging stop, the following properties of the `ev` parameter are
            also required (see documentation for the `ev` parameter):
                + `initialCharge`
                + `maxCharge`
                + `chargingCurve`

                  *Notes*:
                  * This option is only supported for /v8/routes endpoint.
                  * This option is not supported for pass-through waypoints.
                  * If `makeReachable=true` and `minDuration` is not provided (or if `minDuration=0`),
            route calculation may suggest not to charge on this station, for example,
                    if a better station is available in the vicinity.
                  * If `stopDuration` is provided then the total time at the stop is the higher of
            (`stopDuration`, `chargingSetupDuration` + `chargingDuration`).
                  * If charging for `minDuration` would charge above `maxCharge`, the time spent
            charging is capped by `maxCharge`.
                  * If `stopDuration` or `minDuration` exceed `chargingSetupDuration` +
            `chargingDuration`, the `waiting` post action duration is set to the remaining time.
                  * If `makeReachable=true` is not set, the target charge is
            `maxChargeAfterChargingStation`, unless the charging time would be outside of the
            specified duration range.
                    In such case, the nearest valid value is used.
                  * In getRoutesByHandle requests, the target charge from the original route response
            is used, unless the charging time would be outside of the specified duration range.
                    In such case, the nearest valid value is used.
        via (list[str] | Unset):
        departure_time (str | Unset): Specifies the time either as

            * "**RFC 3339**, section 5.6 as defined by either `date-time` or `date-only` 'T'
            `partial-time` (ie no time-offset)", or
            * the special value `any` which stand for unspecified time
        arrival_time (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either
            `date-time` or `date-only` 'T' `partial-time` (ie no time-offset).
        routing_mode (RoutingMode | Unset): Specifies which optimization is applied during the
            calculation.

            * `fast`: Route calculation from start to destination optimized by travel time. In many
              cases, the route returned by `fast` mode may not be the route with the fastest
              possible travel time. For example, the routing service may favor a route that remains on
              a highway, even if a faster travel time can be achieved by taking a detour or shortcut
              through an inconvenient side road.
            * `short`: Route calculation from start to destination disregarding any speed information.
              In this mode, the distance of the route is minimized, while keeping the route sensible.
              This includes, for example, penalizing turns. Because of that, the resulting route will
              not necessarily be the one with minimal distance.

            Notes:
            * The following Transport modes only support `fast` routingMode
              - `bicycle`
              - `bus`
              - `pedestrian`
              - `privateBus`
              - `scooter`
              - `taxi`
        alternatives (int | Unset):  Default: 0.
        avoid (Avoid | Unset):
        allow (Allow | Unset): Explicitly allow features that require users to opt in.
        exclude (Exclude | Unset): Options to exclude strictly during the route calculation.
        units (Units | Unset): Units of measurement used, for example, in guidance instructions.
            The default is `metric`.
        lang (list[str] | Unset):
        return_ (list[Return] | Unset):
        spans (list[Spans] | Unset):
        truck (Truck | Unset): Vehicle-specific parameters
        vehicle (Vehicle | Unset): Vehicle-specific parameters
        consumption_model (str | Unset): Extensible enum: `empirical` `physical` `...`
            Specifies which of the EV consumption models is being used. See the `ev` parameter for
            details on the models.

            * `empirical`
            * `physical`

            **Alpha**: Physical consumption model is in development. It may not be stable and is
            subject to change.
        ev (EVEmpiricalModel | EVPhysicalModel | Unset): EV properties to be used for calculating
            consumption.

            There are two consumption models (`empirical`, `physical`). Set the `consumptionModel`
            parameter to choose which model to use.

            Required `empirical` model properties:
            * `freeFlowSpeedTable`

            Required `physical` model properties:
            * `driveEfficiency`
            * `recuperationEfficiency`

            When using the physical model, certain properties of the `vehicle` parameter are also
            required (see documentation for the `vehicle` parameter for more details):
            * `rollingResistanceCoefficient`
            * `airDragCoefficient`
            * `currentWeight`
            * `frontalArea` or combination of `width` and `height`

            The following `ev` properties are additionally required in order to calculate State-of-
            Charge along the route:
            * `initialCharge`
            * `maxCharge`

            Route reachability based on State-of-Charge will be evaluated for the following
            constraints, if additionally provided,
            * `minChargeAtDestination`
            * `minChargeAtFirstChargingStation`
            * `minChargeAtChargingStation`

            The following properties are additionally required in order to calculate charging at
            charging station waypoints (see documentation for `via` parameter)
            * `chargingCurve`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * EV parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.

            The following properties are additionally required for the router to automatically enhance
            the route with charging stops
            * `makeReachable` set to `true`
            * `chargingCurve`
            * `connectorTypes`
            * `maxChargeAfterChargingStation`
        fuel (Fuel | Unset): **Disclaimer: This parameter is currently in beta release, and is
            therefore subject to breaking changes.**

            Fuel parameters to be used for calculating consumption and related CO2 emission, and toll
            calculation.

            The following attributes are required for calculating consumption:
              * `type`
              * `freeFlowSpeedTable`

            The following attribute is needed for fuel specific toll calculation (if not provided
            default toll instead of fuel specific toll will be returned):
              * `type`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * Fuel parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.
        driver (Driver | Unset): Driver parameters to be used for calculating routes with
            automatically added
            rest stops.
        pedestrianspeed (float | Unset): Pedestrian speed in meters per second
        scooter (Scooter | Unset): Scooter-specific parameters
        currency (str | Unset):
        customizations (list[str] | Unset):
        taxi (Taxi | Unset): Taxi-specific parameters
        tolls (Tolls | Unset): Vehicle-independent options that may affect route toll calculation
            as well as options
            affecting the output of the tolls, such as summaries.

            Since this parameter controls behaviour related to tolls in the return part of the
            response,
            use of this parameter requires `return=tolls` to be selected.
        max_speed_on_segment (str | Unset): A comma separated list of segments with restrictions
            on maximum baseSpeed.

            Each entry has the following structure:
            `{segmentId}(#{direction})?;speed={maxBaseSpeed}`

            The individual parts are:
            * segmentId: The identifier of the referenced topology segment inside the catalog,
            example: `here:cm:segment:207551710`
            * direction (optional): Either '*' for bidirectional (default), '+' for positive
            direction, or '-' for negative direction
            * maxBaseSpeed: New value in m/s of baseSpeed on segment

            Example of a parameter value excluding two segments:
            `here:cm:segment:207551710#+;speed=10,here:cm:segment:76771992;speed=1`

            **Notes**:
            - It does not increase default baseSpeed on segment. If the value is greater than the
            default base speed, then such penalty will have no effect.
            - Minimum valid value for speed is 1
            - Using segments with a modified base speed does not trigger any notifications
            - Maximum number of penalized segments in one request cannot be greater than 1000.
              "penalized segments" refer to segments that have a restrictions on maximum baseSpeed
            with `maxSpeedOnSegment`
              or avoided with `avoid[segments]`
            - In case the same segment is penalized multiple times through values provided in the
            query string and/or the POST body,
              then the most restrictive value will be applied.
        traffic (Traffic | Unset): Traffic specific parameters.
        billing_tag (str | Unset):  Example: ABCD+EFGH.
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AuthErrorResponseSchema | AuthErrorResponseSchema | RoutingErrorResponse | RouterRouteResponse | RoutingErrorResponse]
    """

    kwargs = _get_kwargs(
        transport_mode=transport_mode,
        origin=origin,
        destination=destination,
        via=via,
        departure_time=departure_time,
        arrival_time=arrival_time,
        routing_mode=routing_mode,
        alternatives=alternatives,
        avoid=avoid,
        allow=allow,
        exclude=exclude,
        units=units,
        lang=lang,
        return_=return_,
        spans=spans,
        truck=truck,
        vehicle=vehicle,
        consumption_model=consumption_model,
        ev=ev,
        fuel=fuel,
        driver=driver,
        pedestrianspeed=pedestrianspeed,
        scooter=scooter,
        currency=currency,
        customizations=customizations,
        taxi=taxi,
        tolls=tolls,
        max_speed_on_segment=max_speed_on_segment,
        traffic=traffic,
        billing_tag=billing_tag,
        x_request_id=x_request_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient | Client,
    transport_mode: RouterMode,
    origin: str,
    destination: str,
    via: list[str] | Unset = UNSET,
    departure_time: str | Unset = UNSET,
    arrival_time: datetime.datetime | Unset = UNSET,
    routing_mode: RoutingMode | Unset = UNSET,
    alternatives: int | Unset = 0,
    avoid: Avoid | Unset = UNSET,
    allow: Allow | Unset = UNSET,
    exclude: Exclude | Unset = UNSET,
    units: Units | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    return_: list[Return] | Unset = UNSET,
    spans: list[Spans] | Unset = UNSET,
    truck: Truck | Unset = UNSET,
    vehicle: Vehicle | Unset = UNSET,
    consumption_model: str | Unset = UNSET,
    ev: EVEmpiricalModel | EVPhysicalModel | Unset = UNSET,
    fuel: Fuel | Unset = UNSET,
    driver: Driver | Unset = UNSET,
    pedestrianspeed: float | Unset = UNSET,
    scooter: Scooter | Unset = UNSET,
    currency: str | Unset = UNSET,
    customizations: list[str] | Unset = UNSET,
    taxi: Taxi | Unset = UNSET,
    tolls: Tolls | Unset = UNSET,
    max_speed_on_segment: str | Unset = UNSET,
    traffic: Traffic | Unset = UNSET,
    billing_tag: str | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> (
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | RoutingErrorResponse
    | RouterRouteResponse
    | RoutingErrorResponse
    | None
):
    """Calculate routes via GET

     Calculates a route using a generic vehicle/pedestrian mode, e.g. car, truck, pedestrian, etc...

    Args:
        transport_mode (RouterMode): Mode of transport to be used for route calculation.
        origin (str): A location defining an origin, destination or via point for a route or an
            isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
            * `charging`: Structured string denoting a user-planned charging stop.

                Format: `charging=(power=<value>;current=<value>;voltage=<value>;supplyType=<value>;mi
            nDuration=<value>;maxDuration=<value>)`.

                The properties `power`, `current`, `voltage` and `supplyType` denote the
            characteristics of the chosen compatible connector at the station. These are all required.
                `minDuration` and `maxDuration` set the time bounds for the charging time. At least
            one of them is required. For most use cases we recommend to provide at least
            `minDuration`.
                The following are the specifications for the properties:
                + `power`:        value in kW, type: number. Rated power of the connector.
                + `voltage`:      value in V, type: number. Rated voltage of the connector.
                + `current`:      value in A, type: number. Rated current of the connector.
                + `supplyType`:   one of {"acSingle", "acThree", "dc"} for 1-phase AC, 3-phase AC and
            DC respectively, type: string.
                + `minDuration`:  value in seconds, type: int. Minimum time the user expects to charge
            at the station, including `chargingSetupDuration`.
                + `maxDuration`:  value in seconds, type: int. Maximum time the user plans to charge
            at the station, including `chargingSetupDuration`.

                For a user-planned charging stop, the following properties of the `ev` parameter are
            also required (see documentation for the `ev` parameter):
                + `initialCharge`
                + `maxCharge`
                + `chargingCurve`

                  *Notes*:
                  * This option is only supported for /v8/routes endpoint.
                  * This option is not supported for pass-through waypoints.
                  * If `makeReachable=true` and `minDuration` is not provided (or if `minDuration=0`),
            route calculation may suggest not to charge on this station, for example,
                    if a better station is available in the vicinity.
                  * If `stopDuration` is provided then the total time at the stop is the higher of
            (`stopDuration`, `chargingSetupDuration` + `chargingDuration`).
                  * If charging for `minDuration` would charge above `maxCharge`, the time spent
            charging is capped by `maxCharge`.
                  * If `stopDuration` or `minDuration` exceed `chargingSetupDuration` +
            `chargingDuration`, the `waiting` post action duration is set to the remaining time.
                  * If `makeReachable=true` is not set, the target charge is
            `maxChargeAfterChargingStation`, unless the charging time would be outside of the
            specified duration range.
                    In such case, the nearest valid value is used.
                  * In getRoutesByHandle requests, the target charge from the original route response
            is used, unless the charging time would be outside of the specified duration range.
                    In such case, the nearest valid value is used.
        destination (str): A location defining an origin, destination or via point for a route or
            an isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
            * `charging`: Structured string denoting a user-planned charging stop.

                Format: `charging=(power=<value>;current=<value>;voltage=<value>;supplyType=<value>;mi
            nDuration=<value>;maxDuration=<value>)`.

                The properties `power`, `current`, `voltage` and `supplyType` denote the
            characteristics of the chosen compatible connector at the station. These are all required.
                `minDuration` and `maxDuration` set the time bounds for the charging time. At least
            one of them is required. For most use cases we recommend to provide at least
            `minDuration`.
                The following are the specifications for the properties:
                + `power`:        value in kW, type: number. Rated power of the connector.
                + `voltage`:      value in V, type: number. Rated voltage of the connector.
                + `current`:      value in A, type: number. Rated current of the connector.
                + `supplyType`:   one of {"acSingle", "acThree", "dc"} for 1-phase AC, 3-phase AC and
            DC respectively, type: string.
                + `minDuration`:  value in seconds, type: int. Minimum time the user expects to charge
            at the station, including `chargingSetupDuration`.
                + `maxDuration`:  value in seconds, type: int. Maximum time the user plans to charge
            at the station, including `chargingSetupDuration`.

                For a user-planned charging stop, the following properties of the `ev` parameter are
            also required (see documentation for the `ev` parameter):
                + `initialCharge`
                + `maxCharge`
                + `chargingCurve`

                  *Notes*:
                  * This option is only supported for /v8/routes endpoint.
                  * This option is not supported for pass-through waypoints.
                  * If `makeReachable=true` and `minDuration` is not provided (or if `minDuration=0`),
            route calculation may suggest not to charge on this station, for example,
                    if a better station is available in the vicinity.
                  * If `stopDuration` is provided then the total time at the stop is the higher of
            (`stopDuration`, `chargingSetupDuration` + `chargingDuration`).
                  * If charging for `minDuration` would charge above `maxCharge`, the time spent
            charging is capped by `maxCharge`.
                  * If `stopDuration` or `minDuration` exceed `chargingSetupDuration` +
            `chargingDuration`, the `waiting` post action duration is set to the remaining time.
                  * If `makeReachable=true` is not set, the target charge is
            `maxChargeAfterChargingStation`, unless the charging time would be outside of the
            specified duration range.
                    In such case, the nearest valid value is used.
                  * In getRoutesByHandle requests, the target charge from the original route response
            is used, unless the charging time would be outside of the specified duration range.
                    In such case, the nearest valid value is used.
        via (list[str] | Unset):
        departure_time (str | Unset): Specifies the time either as

            * "**RFC 3339**, section 5.6 as defined by either `date-time` or `date-only` 'T'
            `partial-time` (ie no time-offset)", or
            * the special value `any` which stand for unspecified time
        arrival_time (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either
            `date-time` or `date-only` 'T' `partial-time` (ie no time-offset).
        routing_mode (RoutingMode | Unset): Specifies which optimization is applied during the
            calculation.

            * `fast`: Route calculation from start to destination optimized by travel time. In many
              cases, the route returned by `fast` mode may not be the route with the fastest
              possible travel time. For example, the routing service may favor a route that remains on
              a highway, even if a faster travel time can be achieved by taking a detour or shortcut
              through an inconvenient side road.
            * `short`: Route calculation from start to destination disregarding any speed information.
              In this mode, the distance of the route is minimized, while keeping the route sensible.
              This includes, for example, penalizing turns. Because of that, the resulting route will
              not necessarily be the one with minimal distance.

            Notes:
            * The following Transport modes only support `fast` routingMode
              - `bicycle`
              - `bus`
              - `pedestrian`
              - `privateBus`
              - `scooter`
              - `taxi`
        alternatives (int | Unset):  Default: 0.
        avoid (Avoid | Unset):
        allow (Allow | Unset): Explicitly allow features that require users to opt in.
        exclude (Exclude | Unset): Options to exclude strictly during the route calculation.
        units (Units | Unset): Units of measurement used, for example, in guidance instructions.
            The default is `metric`.
        lang (list[str] | Unset):
        return_ (list[Return] | Unset):
        spans (list[Spans] | Unset):
        truck (Truck | Unset): Vehicle-specific parameters
        vehicle (Vehicle | Unset): Vehicle-specific parameters
        consumption_model (str | Unset): Extensible enum: `empirical` `physical` `...`
            Specifies which of the EV consumption models is being used. See the `ev` parameter for
            details on the models.

            * `empirical`
            * `physical`

            **Alpha**: Physical consumption model is in development. It may not be stable and is
            subject to change.
        ev (EVEmpiricalModel | EVPhysicalModel | Unset): EV properties to be used for calculating
            consumption.

            There are two consumption models (`empirical`, `physical`). Set the `consumptionModel`
            parameter to choose which model to use.

            Required `empirical` model properties:
            * `freeFlowSpeedTable`

            Required `physical` model properties:
            * `driveEfficiency`
            * `recuperationEfficiency`

            When using the physical model, certain properties of the `vehicle` parameter are also
            required (see documentation for the `vehicle` parameter for more details):
            * `rollingResistanceCoefficient`
            * `airDragCoefficient`
            * `currentWeight`
            * `frontalArea` or combination of `width` and `height`

            The following `ev` properties are additionally required in order to calculate State-of-
            Charge along the route:
            * `initialCharge`
            * `maxCharge`

            Route reachability based on State-of-Charge will be evaluated for the following
            constraints, if additionally provided,
            * `minChargeAtDestination`
            * `minChargeAtFirstChargingStation`
            * `minChargeAtChargingStation`

            The following properties are additionally required in order to calculate charging at
            charging station waypoints (see documentation for `via` parameter)
            * `chargingCurve`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * EV parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.

            The following properties are additionally required for the router to automatically enhance
            the route with charging stops
            * `makeReachable` set to `true`
            * `chargingCurve`
            * `connectorTypes`
            * `maxChargeAfterChargingStation`
        fuel (Fuel | Unset): **Disclaimer: This parameter is currently in beta release, and is
            therefore subject to breaking changes.**

            Fuel parameters to be used for calculating consumption and related CO2 emission, and toll
            calculation.

            The following attributes are required for calculating consumption:
              * `type`
              * `freeFlowSpeedTable`

            The following attribute is needed for fuel specific toll calculation (if not provided
            default toll instead of fuel specific toll will be returned):
              * `type`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * Fuel parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.
        driver (Driver | Unset): Driver parameters to be used for calculating routes with
            automatically added
            rest stops.
        pedestrianspeed (float | Unset): Pedestrian speed in meters per second
        scooter (Scooter | Unset): Scooter-specific parameters
        currency (str | Unset):
        customizations (list[str] | Unset):
        taxi (Taxi | Unset): Taxi-specific parameters
        tolls (Tolls | Unset): Vehicle-independent options that may affect route toll calculation
            as well as options
            affecting the output of the tolls, such as summaries.

            Since this parameter controls behaviour related to tolls in the return part of the
            response,
            use of this parameter requires `return=tolls` to be selected.
        max_speed_on_segment (str | Unset): A comma separated list of segments with restrictions
            on maximum baseSpeed.

            Each entry has the following structure:
            `{segmentId}(#{direction})?;speed={maxBaseSpeed}`

            The individual parts are:
            * segmentId: The identifier of the referenced topology segment inside the catalog,
            example: `here:cm:segment:207551710`
            * direction (optional): Either '*' for bidirectional (default), '+' for positive
            direction, or '-' for negative direction
            * maxBaseSpeed: New value in m/s of baseSpeed on segment

            Example of a parameter value excluding two segments:
            `here:cm:segment:207551710#+;speed=10,here:cm:segment:76771992;speed=1`

            **Notes**:
            - It does not increase default baseSpeed on segment. If the value is greater than the
            default base speed, then such penalty will have no effect.
            - Minimum valid value for speed is 1
            - Using segments with a modified base speed does not trigger any notifications
            - Maximum number of penalized segments in one request cannot be greater than 1000.
              "penalized segments" refer to segments that have a restrictions on maximum baseSpeed
            with `maxSpeedOnSegment`
              or avoided with `avoid[segments]`
            - In case the same segment is penalized multiple times through values provided in the
            query string and/or the POST body,
              then the most restrictive value will be applied.
        traffic (Traffic | Unset): Traffic specific parameters.
        billing_tag (str | Unset):  Example: ABCD+EFGH.
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AuthErrorResponseSchema | AuthErrorResponseSchema | RoutingErrorResponse | RouterRouteResponse | RoutingErrorResponse
    """

    return sync_detailed(
        client=client,
        transport_mode=transport_mode,
        origin=origin,
        destination=destination,
        via=via,
        departure_time=departure_time,
        arrival_time=arrival_time,
        routing_mode=routing_mode,
        alternatives=alternatives,
        avoid=avoid,
        allow=allow,
        exclude=exclude,
        units=units,
        lang=lang,
        return_=return_,
        spans=spans,
        truck=truck,
        vehicle=vehicle,
        consumption_model=consumption_model,
        ev=ev,
        fuel=fuel,
        driver=driver,
        pedestrianspeed=pedestrianspeed,
        scooter=scooter,
        currency=currency,
        customizations=customizations,
        taxi=taxi,
        tolls=tolls,
        max_speed_on_segment=max_speed_on_segment,
        traffic=traffic,
        billing_tag=billing_tag,
        x_request_id=x_request_id,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    transport_mode: RouterMode,
    origin: str,
    destination: str,
    via: list[str] | Unset = UNSET,
    departure_time: str | Unset = UNSET,
    arrival_time: datetime.datetime | Unset = UNSET,
    routing_mode: RoutingMode | Unset = UNSET,
    alternatives: int | Unset = 0,
    avoid: Avoid | Unset = UNSET,
    allow: Allow | Unset = UNSET,
    exclude: Exclude | Unset = UNSET,
    units: Units | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    return_: list[Return] | Unset = UNSET,
    spans: list[Spans] | Unset = UNSET,
    truck: Truck | Unset = UNSET,
    vehicle: Vehicle | Unset = UNSET,
    consumption_model: str | Unset = UNSET,
    ev: EVEmpiricalModel | EVPhysicalModel | Unset = UNSET,
    fuel: Fuel | Unset = UNSET,
    driver: Driver | Unset = UNSET,
    pedestrianspeed: float | Unset = UNSET,
    scooter: Scooter | Unset = UNSET,
    currency: str | Unset = UNSET,
    customizations: list[str] | Unset = UNSET,
    taxi: Taxi | Unset = UNSET,
    tolls: Tolls | Unset = UNSET,
    max_speed_on_segment: str | Unset = UNSET,
    traffic: Traffic | Unset = UNSET,
    billing_tag: str | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> Response[
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | RoutingErrorResponse
    | RouterRouteResponse
    | RoutingErrorResponse
]:
    """Calculate routes via GET

     Calculates a route using a generic vehicle/pedestrian mode, e.g. car, truck, pedestrian, etc...

    Args:
        transport_mode (RouterMode): Mode of transport to be used for route calculation.
        origin (str): A location defining an origin, destination or via point for a route or an
            isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
            * `charging`: Structured string denoting a user-planned charging stop.

                Format: `charging=(power=<value>;current=<value>;voltage=<value>;supplyType=<value>;mi
            nDuration=<value>;maxDuration=<value>)`.

                The properties `power`, `current`, `voltage` and `supplyType` denote the
            characteristics of the chosen compatible connector at the station. These are all required.
                `minDuration` and `maxDuration` set the time bounds for the charging time. At least
            one of them is required. For most use cases we recommend to provide at least
            `minDuration`.
                The following are the specifications for the properties:
                + `power`:        value in kW, type: number. Rated power of the connector.
                + `voltage`:      value in V, type: number. Rated voltage of the connector.
                + `current`:      value in A, type: number. Rated current of the connector.
                + `supplyType`:   one of {"acSingle", "acThree", "dc"} for 1-phase AC, 3-phase AC and
            DC respectively, type: string.
                + `minDuration`:  value in seconds, type: int. Minimum time the user expects to charge
            at the station, including `chargingSetupDuration`.
                + `maxDuration`:  value in seconds, type: int. Maximum time the user plans to charge
            at the station, including `chargingSetupDuration`.

                For a user-planned charging stop, the following properties of the `ev` parameter are
            also required (see documentation for the `ev` parameter):
                + `initialCharge`
                + `maxCharge`
                + `chargingCurve`

                  *Notes*:
                  * This option is only supported for /v8/routes endpoint.
                  * This option is not supported for pass-through waypoints.
                  * If `makeReachable=true` and `minDuration` is not provided (or if `minDuration=0`),
            route calculation may suggest not to charge on this station, for example,
                    if a better station is available in the vicinity.
                  * If `stopDuration` is provided then the total time at the stop is the higher of
            (`stopDuration`, `chargingSetupDuration` + `chargingDuration`).
                  * If charging for `minDuration` would charge above `maxCharge`, the time spent
            charging is capped by `maxCharge`.
                  * If `stopDuration` or `minDuration` exceed `chargingSetupDuration` +
            `chargingDuration`, the `waiting` post action duration is set to the remaining time.
                  * If `makeReachable=true` is not set, the target charge is
            `maxChargeAfterChargingStation`, unless the charging time would be outside of the
            specified duration range.
                    In such case, the nearest valid value is used.
                  * In getRoutesByHandle requests, the target charge from the original route response
            is used, unless the charging time would be outside of the specified duration range.
                    In such case, the nearest valid value is used.
        destination (str): A location defining an origin, destination or via point for a route or
            an isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
            * `charging`: Structured string denoting a user-planned charging stop.

                Format: `charging=(power=<value>;current=<value>;voltage=<value>;supplyType=<value>;mi
            nDuration=<value>;maxDuration=<value>)`.

                The properties `power`, `current`, `voltage` and `supplyType` denote the
            characteristics of the chosen compatible connector at the station. These are all required.
                `minDuration` and `maxDuration` set the time bounds for the charging time. At least
            one of them is required. For most use cases we recommend to provide at least
            `minDuration`.
                The following are the specifications for the properties:
                + `power`:        value in kW, type: number. Rated power of the connector.
                + `voltage`:      value in V, type: number. Rated voltage of the connector.
                + `current`:      value in A, type: number. Rated current of the connector.
                + `supplyType`:   one of {"acSingle", "acThree", "dc"} for 1-phase AC, 3-phase AC and
            DC respectively, type: string.
                + `minDuration`:  value in seconds, type: int. Minimum time the user expects to charge
            at the station, including `chargingSetupDuration`.
                + `maxDuration`:  value in seconds, type: int. Maximum time the user plans to charge
            at the station, including `chargingSetupDuration`.

                For a user-planned charging stop, the following properties of the `ev` parameter are
            also required (see documentation for the `ev` parameter):
                + `initialCharge`
                + `maxCharge`
                + `chargingCurve`

                  *Notes*:
                  * This option is only supported for /v8/routes endpoint.
                  * This option is not supported for pass-through waypoints.
                  * If `makeReachable=true` and `minDuration` is not provided (or if `minDuration=0`),
            route calculation may suggest not to charge on this station, for example,
                    if a better station is available in the vicinity.
                  * If `stopDuration` is provided then the total time at the stop is the higher of
            (`stopDuration`, `chargingSetupDuration` + `chargingDuration`).
                  * If charging for `minDuration` would charge above `maxCharge`, the time spent
            charging is capped by `maxCharge`.
                  * If `stopDuration` or `minDuration` exceed `chargingSetupDuration` +
            `chargingDuration`, the `waiting` post action duration is set to the remaining time.
                  * If `makeReachable=true` is not set, the target charge is
            `maxChargeAfterChargingStation`, unless the charging time would be outside of the
            specified duration range.
                    In such case, the nearest valid value is used.
                  * In getRoutesByHandle requests, the target charge from the original route response
            is used, unless the charging time would be outside of the specified duration range.
                    In such case, the nearest valid value is used.
        via (list[str] | Unset):
        departure_time (str | Unset): Specifies the time either as

            * "**RFC 3339**, section 5.6 as defined by either `date-time` or `date-only` 'T'
            `partial-time` (ie no time-offset)", or
            * the special value `any` which stand for unspecified time
        arrival_time (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either
            `date-time` or `date-only` 'T' `partial-time` (ie no time-offset).
        routing_mode (RoutingMode | Unset): Specifies which optimization is applied during the
            calculation.

            * `fast`: Route calculation from start to destination optimized by travel time. In many
              cases, the route returned by `fast` mode may not be the route with the fastest
              possible travel time. For example, the routing service may favor a route that remains on
              a highway, even if a faster travel time can be achieved by taking a detour or shortcut
              through an inconvenient side road.
            * `short`: Route calculation from start to destination disregarding any speed information.
              In this mode, the distance of the route is minimized, while keeping the route sensible.
              This includes, for example, penalizing turns. Because of that, the resulting route will
              not necessarily be the one with minimal distance.

            Notes:
            * The following Transport modes only support `fast` routingMode
              - `bicycle`
              - `bus`
              - `pedestrian`
              - `privateBus`
              - `scooter`
              - `taxi`
        alternatives (int | Unset):  Default: 0.
        avoid (Avoid | Unset):
        allow (Allow | Unset): Explicitly allow features that require users to opt in.
        exclude (Exclude | Unset): Options to exclude strictly during the route calculation.
        units (Units | Unset): Units of measurement used, for example, in guidance instructions.
            The default is `metric`.
        lang (list[str] | Unset):
        return_ (list[Return] | Unset):
        spans (list[Spans] | Unset):
        truck (Truck | Unset): Vehicle-specific parameters
        vehicle (Vehicle | Unset): Vehicle-specific parameters
        consumption_model (str | Unset): Extensible enum: `empirical` `physical` `...`
            Specifies which of the EV consumption models is being used. See the `ev` parameter for
            details on the models.

            * `empirical`
            * `physical`

            **Alpha**: Physical consumption model is in development. It may not be stable and is
            subject to change.
        ev (EVEmpiricalModel | EVPhysicalModel | Unset): EV properties to be used for calculating
            consumption.

            There are two consumption models (`empirical`, `physical`). Set the `consumptionModel`
            parameter to choose which model to use.

            Required `empirical` model properties:
            * `freeFlowSpeedTable`

            Required `physical` model properties:
            * `driveEfficiency`
            * `recuperationEfficiency`

            When using the physical model, certain properties of the `vehicle` parameter are also
            required (see documentation for the `vehicle` parameter for more details):
            * `rollingResistanceCoefficient`
            * `airDragCoefficient`
            * `currentWeight`
            * `frontalArea` or combination of `width` and `height`

            The following `ev` properties are additionally required in order to calculate State-of-
            Charge along the route:
            * `initialCharge`
            * `maxCharge`

            Route reachability based on State-of-Charge will be evaluated for the following
            constraints, if additionally provided,
            * `minChargeAtDestination`
            * `minChargeAtFirstChargingStation`
            * `minChargeAtChargingStation`

            The following properties are additionally required in order to calculate charging at
            charging station waypoints (see documentation for `via` parameter)
            * `chargingCurve`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * EV parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.

            The following properties are additionally required for the router to automatically enhance
            the route with charging stops
            * `makeReachable` set to `true`
            * `chargingCurve`
            * `connectorTypes`
            * `maxChargeAfterChargingStation`
        fuel (Fuel | Unset): **Disclaimer: This parameter is currently in beta release, and is
            therefore subject to breaking changes.**

            Fuel parameters to be used for calculating consumption and related CO2 emission, and toll
            calculation.

            The following attributes are required for calculating consumption:
              * `type`
              * `freeFlowSpeedTable`

            The following attribute is needed for fuel specific toll calculation (if not provided
            default toll instead of fuel specific toll will be returned):
              * `type`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * Fuel parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.
        driver (Driver | Unset): Driver parameters to be used for calculating routes with
            automatically added
            rest stops.
        pedestrianspeed (float | Unset): Pedestrian speed in meters per second
        scooter (Scooter | Unset): Scooter-specific parameters
        currency (str | Unset):
        customizations (list[str] | Unset):
        taxi (Taxi | Unset): Taxi-specific parameters
        tolls (Tolls | Unset): Vehicle-independent options that may affect route toll calculation
            as well as options
            affecting the output of the tolls, such as summaries.

            Since this parameter controls behaviour related to tolls in the return part of the
            response,
            use of this parameter requires `return=tolls` to be selected.
        max_speed_on_segment (str | Unset): A comma separated list of segments with restrictions
            on maximum baseSpeed.

            Each entry has the following structure:
            `{segmentId}(#{direction})?;speed={maxBaseSpeed}`

            The individual parts are:
            * segmentId: The identifier of the referenced topology segment inside the catalog,
            example: `here:cm:segment:207551710`
            * direction (optional): Either '*' for bidirectional (default), '+' for positive
            direction, or '-' for negative direction
            * maxBaseSpeed: New value in m/s of baseSpeed on segment

            Example of a parameter value excluding two segments:
            `here:cm:segment:207551710#+;speed=10,here:cm:segment:76771992;speed=1`

            **Notes**:
            - It does not increase default baseSpeed on segment. If the value is greater than the
            default base speed, then such penalty will have no effect.
            - Minimum valid value for speed is 1
            - Using segments with a modified base speed does not trigger any notifications
            - Maximum number of penalized segments in one request cannot be greater than 1000.
              "penalized segments" refer to segments that have a restrictions on maximum baseSpeed
            with `maxSpeedOnSegment`
              or avoided with `avoid[segments]`
            - In case the same segment is penalized multiple times through values provided in the
            query string and/or the POST body,
              then the most restrictive value will be applied.
        traffic (Traffic | Unset): Traffic specific parameters.
        billing_tag (str | Unset):  Example: ABCD+EFGH.
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AuthErrorResponseSchema | AuthErrorResponseSchema | RoutingErrorResponse | RouterRouteResponse | RoutingErrorResponse]
    """

    kwargs = _get_kwargs(
        transport_mode=transport_mode,
        origin=origin,
        destination=destination,
        via=via,
        departure_time=departure_time,
        arrival_time=arrival_time,
        routing_mode=routing_mode,
        alternatives=alternatives,
        avoid=avoid,
        allow=allow,
        exclude=exclude,
        units=units,
        lang=lang,
        return_=return_,
        spans=spans,
        truck=truck,
        vehicle=vehicle,
        consumption_model=consumption_model,
        ev=ev,
        fuel=fuel,
        driver=driver,
        pedestrianspeed=pedestrianspeed,
        scooter=scooter,
        currency=currency,
        customizations=customizations,
        taxi=taxi,
        tolls=tolls,
        max_speed_on_segment=max_speed_on_segment,
        traffic=traffic,
        billing_tag=billing_tag,
        x_request_id=x_request_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    transport_mode: RouterMode,
    origin: str,
    destination: str,
    via: list[str] | Unset = UNSET,
    departure_time: str | Unset = UNSET,
    arrival_time: datetime.datetime | Unset = UNSET,
    routing_mode: RoutingMode | Unset = UNSET,
    alternatives: int | Unset = 0,
    avoid: Avoid | Unset = UNSET,
    allow: Allow | Unset = UNSET,
    exclude: Exclude | Unset = UNSET,
    units: Units | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    return_: list[Return] | Unset = UNSET,
    spans: list[Spans] | Unset = UNSET,
    truck: Truck | Unset = UNSET,
    vehicle: Vehicle | Unset = UNSET,
    consumption_model: str | Unset = UNSET,
    ev: EVEmpiricalModel | EVPhysicalModel | Unset = UNSET,
    fuel: Fuel | Unset = UNSET,
    driver: Driver | Unset = UNSET,
    pedestrianspeed: float | Unset = UNSET,
    scooter: Scooter | Unset = UNSET,
    currency: str | Unset = UNSET,
    customizations: list[str] | Unset = UNSET,
    taxi: Taxi | Unset = UNSET,
    tolls: Tolls | Unset = UNSET,
    max_speed_on_segment: str | Unset = UNSET,
    traffic: Traffic | Unset = UNSET,
    billing_tag: str | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> (
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | RoutingErrorResponse
    | RouterRouteResponse
    | RoutingErrorResponse
    | None
):
    """Calculate routes via GET

     Calculates a route using a generic vehicle/pedestrian mode, e.g. car, truck, pedestrian, etc...

    Args:
        transport_mode (RouterMode): Mode of transport to be used for route calculation.
        origin (str): A location defining an origin, destination or via point for a route or an
            isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
            * `charging`: Structured string denoting a user-planned charging stop.

                Format: `charging=(power=<value>;current=<value>;voltage=<value>;supplyType=<value>;mi
            nDuration=<value>;maxDuration=<value>)`.

                The properties `power`, `current`, `voltage` and `supplyType` denote the
            characteristics of the chosen compatible connector at the station. These are all required.
                `minDuration` and `maxDuration` set the time bounds for the charging time. At least
            one of them is required. For most use cases we recommend to provide at least
            `minDuration`.
                The following are the specifications for the properties:
                + `power`:        value in kW, type: number. Rated power of the connector.
                + `voltage`:      value in V, type: number. Rated voltage of the connector.
                + `current`:      value in A, type: number. Rated current of the connector.
                + `supplyType`:   one of {"acSingle", "acThree", "dc"} for 1-phase AC, 3-phase AC and
            DC respectively, type: string.
                + `minDuration`:  value in seconds, type: int. Minimum time the user expects to charge
            at the station, including `chargingSetupDuration`.
                + `maxDuration`:  value in seconds, type: int. Maximum time the user plans to charge
            at the station, including `chargingSetupDuration`.

                For a user-planned charging stop, the following properties of the `ev` parameter are
            also required (see documentation for the `ev` parameter):
                + `initialCharge`
                + `maxCharge`
                + `chargingCurve`

                  *Notes*:
                  * This option is only supported for /v8/routes endpoint.
                  * This option is not supported for pass-through waypoints.
                  * If `makeReachable=true` and `minDuration` is not provided (or if `minDuration=0`),
            route calculation may suggest not to charge on this station, for example,
                    if a better station is available in the vicinity.
                  * If `stopDuration` is provided then the total time at the stop is the higher of
            (`stopDuration`, `chargingSetupDuration` + `chargingDuration`).
                  * If charging for `minDuration` would charge above `maxCharge`, the time spent
            charging is capped by `maxCharge`.
                  * If `stopDuration` or `minDuration` exceed `chargingSetupDuration` +
            `chargingDuration`, the `waiting` post action duration is set to the remaining time.
                  * If `makeReachable=true` is not set, the target charge is
            `maxChargeAfterChargingStation`, unless the charging time would be outside of the
            specified duration range.
                    In such case, the nearest valid value is used.
                  * In getRoutesByHandle requests, the target charge from the original route response
            is used, unless the charging time would be outside of the specified duration range.
                    In such case, the nearest valid value is used.
        destination (str): A location defining an origin, destination or via point for a route or
            an isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
            * `charging`: Structured string denoting a user-planned charging stop.

                Format: `charging=(power=<value>;current=<value>;voltage=<value>;supplyType=<value>;mi
            nDuration=<value>;maxDuration=<value>)`.

                The properties `power`, `current`, `voltage` and `supplyType` denote the
            characteristics of the chosen compatible connector at the station. These are all required.
                `minDuration` and `maxDuration` set the time bounds for the charging time. At least
            one of them is required. For most use cases we recommend to provide at least
            `minDuration`.
                The following are the specifications for the properties:
                + `power`:        value in kW, type: number. Rated power of the connector.
                + `voltage`:      value in V, type: number. Rated voltage of the connector.
                + `current`:      value in A, type: number. Rated current of the connector.
                + `supplyType`:   one of {"acSingle", "acThree", "dc"} for 1-phase AC, 3-phase AC and
            DC respectively, type: string.
                + `minDuration`:  value in seconds, type: int. Minimum time the user expects to charge
            at the station, including `chargingSetupDuration`.
                + `maxDuration`:  value in seconds, type: int. Maximum time the user plans to charge
            at the station, including `chargingSetupDuration`.

                For a user-planned charging stop, the following properties of the `ev` parameter are
            also required (see documentation for the `ev` parameter):
                + `initialCharge`
                + `maxCharge`
                + `chargingCurve`

                  *Notes*:
                  * This option is only supported for /v8/routes endpoint.
                  * This option is not supported for pass-through waypoints.
                  * If `makeReachable=true` and `minDuration` is not provided (or if `minDuration=0`),
            route calculation may suggest not to charge on this station, for example,
                    if a better station is available in the vicinity.
                  * If `stopDuration` is provided then the total time at the stop is the higher of
            (`stopDuration`, `chargingSetupDuration` + `chargingDuration`).
                  * If charging for `minDuration` would charge above `maxCharge`, the time spent
            charging is capped by `maxCharge`.
                  * If `stopDuration` or `minDuration` exceed `chargingSetupDuration` +
            `chargingDuration`, the `waiting` post action duration is set to the remaining time.
                  * If `makeReachable=true` is not set, the target charge is
            `maxChargeAfterChargingStation`, unless the charging time would be outside of the
            specified duration range.
                    In such case, the nearest valid value is used.
                  * In getRoutesByHandle requests, the target charge from the original route response
            is used, unless the charging time would be outside of the specified duration range.
                    In such case, the nearest valid value is used.
        via (list[str] | Unset):
        departure_time (str | Unset): Specifies the time either as

            * "**RFC 3339**, section 5.6 as defined by either `date-time` or `date-only` 'T'
            `partial-time` (ie no time-offset)", or
            * the special value `any` which stand for unspecified time
        arrival_time (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either
            `date-time` or `date-only` 'T' `partial-time` (ie no time-offset).
        routing_mode (RoutingMode | Unset): Specifies which optimization is applied during the
            calculation.

            * `fast`: Route calculation from start to destination optimized by travel time. In many
              cases, the route returned by `fast` mode may not be the route with the fastest
              possible travel time. For example, the routing service may favor a route that remains on
              a highway, even if a faster travel time can be achieved by taking a detour or shortcut
              through an inconvenient side road.
            * `short`: Route calculation from start to destination disregarding any speed information.
              In this mode, the distance of the route is minimized, while keeping the route sensible.
              This includes, for example, penalizing turns. Because of that, the resulting route will
              not necessarily be the one with minimal distance.

            Notes:
            * The following Transport modes only support `fast` routingMode
              - `bicycle`
              - `bus`
              - `pedestrian`
              - `privateBus`
              - `scooter`
              - `taxi`
        alternatives (int | Unset):  Default: 0.
        avoid (Avoid | Unset):
        allow (Allow | Unset): Explicitly allow features that require users to opt in.
        exclude (Exclude | Unset): Options to exclude strictly during the route calculation.
        units (Units | Unset): Units of measurement used, for example, in guidance instructions.
            The default is `metric`.
        lang (list[str] | Unset):
        return_ (list[Return] | Unset):
        spans (list[Spans] | Unset):
        truck (Truck | Unset): Vehicle-specific parameters
        vehicle (Vehicle | Unset): Vehicle-specific parameters
        consumption_model (str | Unset): Extensible enum: `empirical` `physical` `...`
            Specifies which of the EV consumption models is being used. See the `ev` parameter for
            details on the models.

            * `empirical`
            * `physical`

            **Alpha**: Physical consumption model is in development. It may not be stable and is
            subject to change.
        ev (EVEmpiricalModel | EVPhysicalModel | Unset): EV properties to be used for calculating
            consumption.

            There are two consumption models (`empirical`, `physical`). Set the `consumptionModel`
            parameter to choose which model to use.

            Required `empirical` model properties:
            * `freeFlowSpeedTable`

            Required `physical` model properties:
            * `driveEfficiency`
            * `recuperationEfficiency`

            When using the physical model, certain properties of the `vehicle` parameter are also
            required (see documentation for the `vehicle` parameter for more details):
            * `rollingResistanceCoefficient`
            * `airDragCoefficient`
            * `currentWeight`
            * `frontalArea` or combination of `width` and `height`

            The following `ev` properties are additionally required in order to calculate State-of-
            Charge along the route:
            * `initialCharge`
            * `maxCharge`

            Route reachability based on State-of-Charge will be evaluated for the following
            constraints, if additionally provided,
            * `minChargeAtDestination`
            * `minChargeAtFirstChargingStation`
            * `minChargeAtChargingStation`

            The following properties are additionally required in order to calculate charging at
            charging station waypoints (see documentation for `via` parameter)
            * `chargingCurve`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * EV parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.

            The following properties are additionally required for the router to automatically enhance
            the route with charging stops
            * `makeReachable` set to `true`
            * `chargingCurve`
            * `connectorTypes`
            * `maxChargeAfterChargingStation`
        fuel (Fuel | Unset): **Disclaimer: This parameter is currently in beta release, and is
            therefore subject to breaking changes.**

            Fuel parameters to be used for calculating consumption and related CO2 emission, and toll
            calculation.

            The following attributes are required for calculating consumption:
              * `type`
              * `freeFlowSpeedTable`

            The following attribute is needed for fuel specific toll calculation (if not provided
            default toll instead of fuel specific toll will be returned):
              * `type`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * Fuel parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.
        driver (Driver | Unset): Driver parameters to be used for calculating routes with
            automatically added
            rest stops.
        pedestrianspeed (float | Unset): Pedestrian speed in meters per second
        scooter (Scooter | Unset): Scooter-specific parameters
        currency (str | Unset):
        customizations (list[str] | Unset):
        taxi (Taxi | Unset): Taxi-specific parameters
        tolls (Tolls | Unset): Vehicle-independent options that may affect route toll calculation
            as well as options
            affecting the output of the tolls, such as summaries.

            Since this parameter controls behaviour related to tolls in the return part of the
            response,
            use of this parameter requires `return=tolls` to be selected.
        max_speed_on_segment (str | Unset): A comma separated list of segments with restrictions
            on maximum baseSpeed.

            Each entry has the following structure:
            `{segmentId}(#{direction})?;speed={maxBaseSpeed}`

            The individual parts are:
            * segmentId: The identifier of the referenced topology segment inside the catalog,
            example: `here:cm:segment:207551710`
            * direction (optional): Either '*' for bidirectional (default), '+' for positive
            direction, or '-' for negative direction
            * maxBaseSpeed: New value in m/s of baseSpeed on segment

            Example of a parameter value excluding two segments:
            `here:cm:segment:207551710#+;speed=10,here:cm:segment:76771992;speed=1`

            **Notes**:
            - It does not increase default baseSpeed on segment. If the value is greater than the
            default base speed, then such penalty will have no effect.
            - Minimum valid value for speed is 1
            - Using segments with a modified base speed does not trigger any notifications
            - Maximum number of penalized segments in one request cannot be greater than 1000.
              "penalized segments" refer to segments that have a restrictions on maximum baseSpeed
            with `maxSpeedOnSegment`
              or avoided with `avoid[segments]`
            - In case the same segment is penalized multiple times through values provided in the
            query string and/or the POST body,
              then the most restrictive value will be applied.
        traffic (Traffic | Unset): Traffic specific parameters.
        billing_tag (str | Unset):  Example: ABCD+EFGH.
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AuthErrorResponseSchema | AuthErrorResponseSchema | RoutingErrorResponse | RouterRouteResponse | RoutingErrorResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            transport_mode=transport_mode,
            origin=origin,
            destination=destination,
            via=via,
            departure_time=departure_time,
            arrival_time=arrival_time,
            routing_mode=routing_mode,
            alternatives=alternatives,
            avoid=avoid,
            allow=allow,
            exclude=exclude,
            units=units,
            lang=lang,
            return_=return_,
            spans=spans,
            truck=truck,
            vehicle=vehicle,
            consumption_model=consumption_model,
            ev=ev,
            fuel=fuel,
            driver=driver,
            pedestrianspeed=pedestrianspeed,
            scooter=scooter,
            currency=currency,
            customizations=customizations,
            taxi=taxi,
            tolls=tolls,
            max_speed_on_segment=max_speed_on_segment,
            traffic=traffic,
            billing_tag=billing_tag,
            x_request_id=x_request_id,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.postal_code_details_usps_zip_postal_code_type import (
    PostalCodeDetailsUspsZipPostalCodeType,
)
from ..models.postal_code_details_usps_zip_postal_entity import (
    PostalCodeDetailsUspsZipPostalEntity,
)
from ..models.postal_code_details_usps_zip_zip_classification_code import (
    PostalCodeDetailsUspsZipZipClassificationCode,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="PostalCodeDetailsUspsZip")


@_attrs_define
class PostalCodeDetailsUspsZip:
    """
    Attributes:
        postal_code (str): The postal code.
        postal_entity (PostalCodeDetailsUspsZipPostalEntity | Unset): The postal entity. This could be a governmental
            authority, a regulatory authority, or a designated postal operator.

            Description of supported values:

            - `USPS`: The USPS postal code system.
        postal_code_type (PostalCodeDetailsUspsZipPostalCodeType | Unset): The postal code type. Currently supported
            values: `ZIP` and `ZIP+4`

            Description of supported values:

            - `ZIP`: A 5-digit code that identifies a specific geographic delivery area. ZIP codes can represent an area
            within a state, or a single building or company that has a very high mail volume.
        zip_classification_code (PostalCodeDetailsUspsZipZipClassificationCode | Unset): The ZIP Classification Code.

            Description of supported values:

            - `M`: Military ZIP code.
            - `P`: ZIP code having only Post Office boxes.
            - `U`: Unique ZIP code: A ZIP assigned to a company, agency, or entity with sufficient mail volume, based on
            average daily volume of letter-size pieces.
    """

    postal_code: str
    postal_entity: PostalCodeDetailsUspsZipPostalEntity | Unset = UNSET
    postal_code_type: PostalCodeDetailsUspsZipPostalCodeType | Unset = UNSET
    zip_classification_code: PostalCodeDetailsUspsZipZipClassificationCode | Unset = (
        UNSET
    )
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        postal_code = self.postal_code

        postal_entity: str | Unset = UNSET
        if not isinstance(self.postal_entity, Unset):
            postal_entity = self.postal_entity.value

        postal_code_type: str | Unset = UNSET
        if not isinstance(self.postal_code_type, Unset):
            postal_code_type = self.postal_code_type.value

        zip_classification_code: str | Unset = UNSET
        if not isinstance(self.zip_classification_code, Unset):
            zip_classification_code = self.zip_classification_code.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "postalCode": postal_code,
        })
        if postal_entity is not UNSET:
            field_dict["postalEntity"] = postal_entity
        if postal_code_type is not UNSET:
            field_dict["postalCodeType"] = postal_code_type
        if zip_classification_code is not UNSET:
            field_dict["zipClassificationCode"] = zip_classification_code

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        postal_code = d.pop("postalCode")

        _postal_entity = d.pop("postalEntity", UNSET)
        postal_entity: PostalCodeDetailsUspsZipPostalEntity | Unset
        if isinstance(_postal_entity, Unset):
            postal_entity = UNSET
        else:
            postal_entity = PostalCodeDetailsUspsZipPostalEntity(_postal_entity)

        _postal_code_type = d.pop("postalCodeType", UNSET)
        postal_code_type: PostalCodeDetailsUspsZipPostalCodeType | Unset
        if isinstance(_postal_code_type, Unset):
            postal_code_type = UNSET
        else:
            postal_code_type = PostalCodeDetailsUspsZipPostalCodeType(_postal_code_type)

        _zip_classification_code = d.pop("zipClassificationCode", UNSET)
        zip_classification_code: PostalCodeDetailsUspsZipZipClassificationCode | Unset
        if isinstance(_zip_classification_code, Unset):
            zip_classification_code = UNSET
        else:
            zip_classification_code = PostalCodeDetailsUspsZipZipClassificationCode(
                _zip_classification_code
            )

        postal_code_details_usps_zip = cls(
            postal_code=postal_code,
            postal_entity=postal_entity,
            postal_code_type=postal_code_type,
            zip_classification_code=zip_classification_code,
        )

        postal_code_details_usps_zip.additional_properties = d
        return postal_code_details_usps_zip

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.functional_class_value import FunctionalClassValue

T = TypeVar("T", bound="FunctionalClass")


@_attrs_define
class FunctionalClass:
    """
    Attributes:
        value (FunctionalClassValue):

            Description of supported values:

            - `1`: allowing for high volume, maximum speed traffic movement
            - `2`: allowing for high volume, high speed traffic movement
            - `3`: providing a high volume of traffic movement
            - `4`: providing for a high volume of traffic movement at moderate speeds between neighbourhoods
            - `5`: roads whose volume and traffic movement are below the level of any functional class
    """

    value: FunctionalClassValue
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        value = self.value.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "value": value,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        value = FunctionalClassValue(d.pop("value"))

        functional_class = cls(
            value=value,
        )

        functional_class.additional_properties = d
        return functional_class

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TrafficIncidentCriticality(str, Enum):
    CRITICAL = "critical"
    LOW = "low"
    MAJOR = "major"
    MINOR = "minor"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class LocalizedRouteNumberDirection(str, Enum):
    EAST = "east"
    NORTH = "north"
    SOUTH = "south"
    WEST = "west"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="RgcAddress")


@_attrs_define
class RgcAddress:
    """
    Attributes:
        label (str | Unset): Assembled address value built out of the address components according to the regional
            postal rules.
            These are the same rules for all endpoints. It may not include all the input terms. For example:
            "Schulstraße 4, 32547 Bad Oeynhausen, Germany"
        country_code (str | Unset): A three-letter country code. For example: "DEU"
        country_name (str | Unset): The localised country name. For example: "Deutschland"
        state_code (str | Unset): A state code or state name abbreviation – country specific. For example, in the United
            States it is the two letter state abbreviation: "CA" for California.
        state (str | Unset): The state division of a country. For example: "North Rhine-Westphalia"
        county_code (str | Unset): A county code or county name abbreviation – country specific. For example, for Italy
            it is the province abbreviation: "RM" for Rome.
        county (str | Unset): A division of a state; typically, a secondary-level administrative division of a country
            or equivalent.
        city (str | Unset): The name of the primary locality of the place. For example: "Bad Oyenhausen"
        district (str | Unset): A division of city; typically an administrative unit within a larger city or a customary
            name of a city's neighborhood. For example: "Bad Oyenhausen"
        subdistrict (str | Unset): A subdivision of a district. For example: "Minden-Lübbecke"
        street (str | Unset): Name of street. For example: "Schulstrasse"
        block (str | Unset): Name of block.
        subblock (str | Unset): Name of sub-block.
        postal_code (str | Unset): An alphanumeric string included in a postal address to facilitate mail sorting, such
            as post code, postcode, or ZIP code. For example: "32547"
        house_number (str | Unset): House number. For example: "4"
        building (str | Unset): Name of building.
        unit (str | Unset): Secondary unit information. It may include building, floor (level), and suite (unit)
            details. This field is returned by Geocode, (Multi) Reverse Geocode and Lookup endpoints only.
    """

    label: str | Unset = UNSET
    country_code: str | Unset = UNSET
    country_name: str | Unset = UNSET
    state_code: str | Unset = UNSET
    state: str | Unset = UNSET
    county_code: str | Unset = UNSET
    county: str | Unset = UNSET
    city: str | Unset = UNSET
    district: str | Unset = UNSET
    subdistrict: str | Unset = UNSET
    street: str | Unset = UNSET
    block: str | Unset = UNSET
    subblock: str | Unset = UNSET
    postal_code: str | Unset = UNSET
    house_number: str | Unset = UNSET
    building: str | Unset = UNSET
    unit: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        label = self.label

        country_code = self.country_code

        country_name = self.country_name

        state_code = self.state_code

        state = self.state

        county_code = self.county_code

        county = self.county

        city = self.city

        district = self.district

        subdistrict = self.subdistrict

        street = self.street

        block = self.block

        subblock = self.subblock

        postal_code = self.postal_code

        house_number = self.house_number

        building = self.building

        unit = self.unit

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if label is not UNSET:
            field_dict["label"] = label
        if country_code is not UNSET:
            field_dict["countryCode"] = country_code
        if country_name is not UNSET:
            field_dict["countryName"] = country_name
        if state_code is not UNSET:
            field_dict["stateCode"] = state_code
        if state is not UNSET:
            field_dict["state"] = state
        if county_code is not UNSET:
            field_dict["countyCode"] = county_code
        if county is not UNSET:
            field_dict["county"] = county
        if city is not UNSET:
            field_dict["city"] = city
        if district is not UNSET:
            field_dict["district"] = district
        if subdistrict is not UNSET:
            field_dict["subdistrict"] = subdistrict
        if street is not UNSET:
            field_dict["street"] = street
        if block is not UNSET:
            field_dict["block"] = block
        if subblock is not UNSET:
            field_dict["subblock"] = subblock
        if postal_code is not UNSET:
            field_dict["postalCode"] = postal_code
        if house_number is not UNSET:
            field_dict["houseNumber"] = house_number
        if building is not UNSET:
            field_dict["building"] = building
        if unit is not UNSET:
            field_dict["unit"] = unit

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        label = d.pop("label", UNSET)

        country_code = d.pop("countryCode", UNSET)

        country_name = d.pop("countryName", UNSET)

        state_code = d.pop("stateCode", UNSET)

        state = d.pop("state", UNSET)

        county_code = d.pop("countyCode", UNSET)

        county = d.pop("county", UNSET)

        city = d.pop("city", UNSET)

        district = d.pop("district", UNSET)

        subdistrict = d.pop("subdistrict", UNSET)

        street = d.pop("street", UNSET)

        block = d.pop("block", UNSET)

        subblock = d.pop("subblock", UNSET)

        postal_code = d.pop("postalCode", UNSET)

        house_number = d.pop("houseNumber", UNSET)

        building = d.pop("building", UNSET)

        unit = d.pop("unit", UNSET)

        rgc_address = cls(
            label=label,
            country_code=country_code,
            country_name=country_name,
            state_code=state_code,
            state=state,
            county_code=county_code,
            county=county,
            city=city,
            district=district,
            subdistrict=subdistrict,
            street=street,
            block=block,
            subblock=subblock,
            postal_code=postal_code,
            house_number=house_number,
            building=building,
            unit=unit,
        )

        rgc_address.additional_properties = d
        return rgc_address

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.ev_charging_attributes_access import EvChargingAttributesAccess
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.e_mobility_service_provider import EMobilityServiceProvider
    from ..models.ev_connector import EvConnector
    from ..models.ev_payment_support import EvPaymentSupport


T = TypeVar("T", bound="EvChargingAttributes")


@_attrs_define
class EvChargingAttributes:
    """
    Attributes:
        connectors (list[EvConnector] | Unset): List of EV pool groups of connectors. Each group is defined by a common
            charging connector type and max power level. The numberOfConnectors field contains the number of connectors in
            the group.
        total_number_of_connectors (int | Unset): Total number of charging connectors in the EV charging pool
        access (EvChargingAttributesAccess | Unset): Information about accessibility
        e_mobility_service_providers (list[EMobilityServiceProvider] | Unset): The list of eMSP (e-Mobility Service
            Provider) for which the EV station operator has EV roaming agreements.
            Each element contains both the name and the partner ID of the eMSP.
        payment_methods (list[EvPaymentSupport] | Unset): List of payment methods supported at the EV station
    """

    connectors: list[EvConnector] | Unset = UNSET
    total_number_of_connectors: int | Unset = UNSET
    access: EvChargingAttributesAccess | Unset = UNSET
    e_mobility_service_providers: list[EMobilityServiceProvider] | Unset = UNSET
    payment_methods: list[EvPaymentSupport] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        connectors: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.connectors, Unset):
            connectors = []
            for connectors_item_data in self.connectors:
                connectors_item = connectors_item_data.to_dict()
                connectors.append(connectors_item)

        total_number_of_connectors = self.total_number_of_connectors

        access: str | Unset = UNSET
        if not isinstance(self.access, Unset):
            access = self.access.value

        e_mobility_service_providers: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.e_mobility_service_providers, Unset):
            e_mobility_service_providers = []
            for (
                e_mobility_service_providers_item_data
            ) in self.e_mobility_service_providers:
                e_mobility_service_providers_item = (
                    e_mobility_service_providers_item_data.to_dict()
                )
                e_mobility_service_providers.append(e_mobility_service_providers_item)

        payment_methods: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.payment_methods, Unset):
            payment_methods = []
            for payment_methods_item_data in self.payment_methods:
                payment_methods_item = payment_methods_item_data.to_dict()
                payment_methods.append(payment_methods_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if connectors is not UNSET:
            field_dict["connectors"] = connectors
        if total_number_of_connectors is not UNSET:
            field_dict["totalNumberOfConnectors"] = total_number_of_connectors
        if access is not UNSET:
            field_dict["access"] = access
        if e_mobility_service_providers is not UNSET:
            field_dict["eMobilityServiceProviders"] = e_mobility_service_providers
        if payment_methods is not UNSET:
            field_dict["paymentMethods"] = payment_methods

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.e_mobility_service_provider import EMobilityServiceProvider
        from ..models.ev_connector import EvConnector
        from ..models.ev_payment_support import EvPaymentSupport

        d = dict(src_dict)
        _connectors = d.pop("connectors", UNSET)
        connectors: list[EvConnector] | Unset = UNSET
        if _connectors is not UNSET:
            connectors = []
            for connectors_item_data in _connectors:
                connectors_item = EvConnector.from_dict(connectors_item_data)

                connectors.append(connectors_item)

        total_number_of_connectors = d.pop("totalNumberOfConnectors", UNSET)

        _access = d.pop("access", UNSET)
        access: EvChargingAttributesAccess | Unset
        if isinstance(_access, Unset):
            access = UNSET
        else:
            access = EvChargingAttributesAccess(_access)

        _e_mobility_service_providers = d.pop("eMobilityServiceProviders", UNSET)
        e_mobility_service_providers: list[EMobilityServiceProvider] | Unset = UNSET
        if _e_mobility_service_providers is not UNSET:
            e_mobility_service_providers = []
            for e_mobility_service_providers_item_data in _e_mobility_service_providers:
                e_mobility_service_providers_item = EMobilityServiceProvider.from_dict(
                    e_mobility_service_providers_item_data
                )

                e_mobility_service_providers.append(e_mobility_service_providers_item)

        _payment_methods = d.pop("paymentMethods", UNSET)
        payment_methods: list[EvPaymentSupport] | Unset = UNSET
        if _payment_methods is not UNSET:
            payment_methods = []
            for payment_methods_item_data in _payment_methods:
                payment_methods_item = EvPaymentSupport.from_dict(
                    payment_methods_item_data
                )

                payment_methods.append(payment_methods_item)

        ev_charging_attributes = cls(
            connectors=connectors,
            total_number_of_connectors=total_number_of_connectors,
            access=access,
            e_mobility_service_providers=e_mobility_service_providers,
            payment_methods=payment_methods,
        )

        ev_charging_attributes.additional_properties = d
        return ev_charging_attributes

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Scooter")


@_attrs_define
class Scooter:
    """Scooter-specific parameters

    Attributes:
        allow_highway (bool | Unset): Specifies whether the scooter is allowed on the highway or not. This parameter is
            optional. If not provided, the scooter is not allowed to use the highway by default. There is a similar
            parameter avoid[features]=controlledAccessHighway to disallow highway usage. avoid[features] takes precedence,
            so if this parameter is also used, scooters are not allowed to use highways even if `allowHighway` is set to
            `true`. Default: False.
    """

    allow_highway: bool | Unset = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        allow_highway = self.allow_highway

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if allow_highway is not UNSET:
            field_dict["allowHighway"] = allow_highway

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        allow_highway = d.pop("allowHighway", UNSET)

        scooter = cls(
            allow_highway=allow_highway,
        )

        scooter.additional_properties = d
        return scooter

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class RelatedAddressResultType(str, Enum):
    ADDRESSBLOCK = "addressBlock"
    ADMINISTRATIVEAREA = "administrativeArea"
    HOUSENUMBER = "houseNumber"
    INTERSECTION = "intersection"
    LOCALITY = "locality"
    PLACE = "place"
    POSTALCODEPOINT = "postalCodePoint"
    STREET = "street"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.charging_action_action import ChargingActionAction
from ..types import UNSET, Unset

T = TypeVar("T", bound="ChargingAction")


@_attrs_define
class ChargingAction:
    """An action to charge the vehicle.

    Attributes:
        action (ChargingActionAction): The type of the action.

            **NOTE:** The list of possible actions may be extended in the future. The client application should handle such
            a case gracefully.
        duration (int): Duration in seconds. Example: 198.
        instruction (str | Unset): Description of the action (e.g. Turn left onto Minna St.).
        consumable_power (float | Unset): Maximum charging power (in kW) available to the vehicle, based on the
            properties of the charging
            station and the vehicle.
        arrival_charge (float | Unset): Estimated vehicle battery charge before this action (in kWh).
        target_charge (float | Unset): Level to which vehicle battery should be charged by this action (in kWh).
    """

    action: ChargingActionAction
    duration: int
    instruction: str | Unset = UNSET
    consumable_power: float | Unset = UNSET
    arrival_charge: float | Unset = UNSET
    target_charge: float | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        action = self.action.value

        duration = self.duration

        instruction = self.instruction

        consumable_power = self.consumable_power

        arrival_charge = self.arrival_charge

        target_charge = self.target_charge

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "action": action,
            "duration": duration,
        })
        if instruction is not UNSET:
            field_dict["instruction"] = instruction
        if consumable_power is not UNSET:
            field_dict["consumablePower"] = consumable_power
        if arrival_charge is not UNSET:
            field_dict["arrivalCharge"] = arrival_charge
        if target_charge is not UNSET:
            field_dict["targetCharge"] = target_charge

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        action = ChargingActionAction(d.pop("action"))

        duration = d.pop("duration")

        instruction = d.pop("instruction", UNSET)

        consumable_power = d.pop("consumablePower", UNSET)

        arrival_charge = d.pop("arrivalCharge", UNSET)

        target_charge = d.pop("targetCharge", UNSET)

        charging_action = cls(
            action=action,
            duration=duration,
            instruction=instruction,
            consumable_power=consumable_power,
            arrival_charge=arrival_charge,
            target_charge=target_charge,
        )

        charging_action.additional_properties = d
        return charging_action

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="SecondaryUnitInfo")


@_attrs_define
class SecondaryUnitInfo:
    """
    Attributes:
        normalized_unit_type (str): The secondary unit designator is provided in a normalized, country-specific format.
            If the secondary unit designator is omitted or unrecognized, it is returned as "unknown".
        unit_value (str): The unit value in normalized form. It may be empty when secondary unit designator has no unit
            value, for example "Basement".
    """

    normalized_unit_type: str
    unit_value: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        normalized_unit_type = self.normalized_unit_type

        unit_value = self.unit_value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "normalizedUnitType": normalized_unit_type,
            "unitValue": unit_value,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        normalized_unit_type = d.pop("normalizedUnitType")

        unit_value = d.pop("unitValue")

        secondary_unit_info = cls(
            normalized_unit_type=normalized_unit_type,
            unit_value=unit_value,
        )

        secondary_unit_info.additional_properties = d
        return secondary_unit_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class GetRevgeocodeWithItem(str, Enum):
    ESTIMATEDAREAFALLBACK = "estimatedAreaFallback"
    MPA = "MPA"
    UNNAMEDSTREETS = "unnamedStreets"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.ev_availability_evse_state import EvAvailabilityEvseState
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.ev_availability_connector import EvAvailabilityConnector


T = TypeVar("T", bound="EvAvailabilityEvse")


@_attrs_define
class EvAvailabilityEvse:
    """
    Attributes:
        id (str | Unset): HERE ID of the EVSE
        cpo_id (str | Unset): The unique ID of an EVSE in the system of the CPO.
             This ID is unique in the system of the CPO but not necessarily globally unique.
             The format will differ between different CPOs. This ID is always provided.
        cpo_evse_emi3_id (str | Unset): Identifier in eMI3 format of the EVSE within the Charge Point Operator (CPO)
            platform.
            This ID is not always present. Example of ID format: "`DE*ICT*E0001897`".
        state (EvAvailabilityEvseState | Unset): EVSE status

            Description of supported values:

            - `AVAILABLE`: The EVSE is able to start a new charging session.
            - `OCCUPIED`: The EVSE is in use
            - `OFFLINE`: No status information available. (Also used when offline)
            - `OTHER`: No status information available. (Also used when offline)
            - `OUT_OF_SERVICE`: The EVSE is currently out of order.
            - `RESERVED`: The EVSE has been reserved for a particular EV driver and is unavailable for other drivers
            - `UNAVAILABLE`: The EVSE is not available because of a physical barrier, for example a car
        last_updated (str | Unset): Last update of the dynamic connector availability information reflected in the
            connectorStatus elements,
            in ISO 8601 format. If the time is UTC, a Z is added.
            An example of this kind of timestamp value is "`2013-12-31T12:00:00.000Z`". If the time is not UTC ,
            then the offset is added as a ±[hh][mm] value (for example, "`2014-01-14T10:00:00.000+0100`").
        connectors (list[EvAvailabilityConnector] | Unset): List of connectors of this EVSE.
    """

    id: str | Unset = UNSET
    cpo_id: str | Unset = UNSET
    cpo_evse_emi3_id: str | Unset = UNSET
    state: EvAvailabilityEvseState | Unset = UNSET
    last_updated: str | Unset = UNSET
    connectors: list[EvAvailabilityConnector] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        cpo_id = self.cpo_id

        cpo_evse_emi3_id = self.cpo_evse_emi3_id

        state: str | Unset = UNSET
        if not isinstance(self.state, Unset):
            state = self.state.value

        last_updated = self.last_updated

        connectors: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.connectors, Unset):
            connectors = []
            for connectors_item_data in self.connectors:
                connectors_item = connectors_item_data.to_dict()
                connectors.append(connectors_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if cpo_id is not UNSET:
            field_dict["cpoId"] = cpo_id
        if cpo_evse_emi3_id is not UNSET:
            field_dict["cpoEvseEMI3Id"] = cpo_evse_emi3_id
        if state is not UNSET:
            field_dict["state"] = state
        if last_updated is not UNSET:
            field_dict["last_updated"] = last_updated
        if connectors is not UNSET:
            field_dict["connectors"] = connectors

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.ev_availability_connector import EvAvailabilityConnector

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        cpo_id = d.pop("cpoId", UNSET)

        cpo_evse_emi3_id = d.pop("cpoEvseEMI3Id", UNSET)

        _state = d.pop("state", UNSET)
        state: EvAvailabilityEvseState | Unset
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = EvAvailabilityEvseState(_state)

        last_updated = d.pop("last_updated", UNSET)

        _connectors = d.pop("connectors", UNSET)
        connectors: list[EvAvailabilityConnector] | Unset = UNSET
        if _connectors is not UNSET:
            connectors = []
            for connectors_item_data in _connectors:
                connectors_item = EvAvailabilityConnector.from_dict(
                    connectors_item_data
                )

                connectors.append(connectors_item)

        ev_availability_evse = cls(
            id=id,
            cpo_id=cpo_id,
            cpo_evse_emi3_id=cpo_evse_emi3_id,
            state=state,
            last_updated=last_updated,
            connectors=connectors,
        )

        ev_availability_evse.additional_properties = d
        return ev_availability_evse

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class GetGeocodeTypesItem(str, Enum):
    ADDRESS = "address"
    AREA = "area"
    CITY = "city"
    HOUSENUMBER = "houseNumber"
    PLACE = "place"
    POSTALCODE = "postalCode"
    STREET = "street"

    def __str__(self) -> str:
        return str(self.value)
